/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2022                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBEGUI_REMOTEFILEDIALOG_H
#define CUBEGUI_REMOTEFILEDIALOG_H

#include <QDialog>
#include <QStack>
#include <QTreeView>
#include "RemoteFileSystemModel.h"

namespace cubegui
{
class PathWidget : public QWidget
{
    Q_OBJECT
public:
    PathWidget( const QString& path );

    void
    setPath( const QString& path );

    QString
    getPath() const;

signals:
    void
        pathChanged( QString );

private:
    QString         path;
    QString         prevPath;  // path before having selected a parent
    QStack<QString> forwards;
    QStack<QString> backwards;
    bool            buttonsUsed    = false; // action triggered by back or forward button
    bool            parentSelected = false; // action triggered by path button

    QStringList
    getPathList( const QString& pathStr,
                 const QString& currentPath );
};

class UrlWidget : public QWidget
{
    Q_OBJECT
public:
    UrlWidget( const QString& url );

    void
    setUrl( const QString& url );

    QString
    getUrl() const;

signals:
    void
        urlChanged( QString );
private:
    QLineEdit* serverLine;
    QLineEdit* portLine;
};

class RemoteFileDialog : public QDialog
{
public:
    /**
     * @brief getFileName opens a connection to a cube server and
     * @param directory initial directory to show on the file dialog. If empty, the directory where the cube-server is started is choosen.
     * @param parent parent widget of the file dialog
     * @return filename, that the user has choosen or empty string if cancelled
     */
    static QString
    getFileName( const QString& directory = "",
                 QWidget*       parent = NULL );

private:
    RemoteFileDialog( const QString& server,
                      const QString& directory = ".",
                      QWidget*       parent = NULL );
    void
    fileSelected( QString filename );
    void
    markHistory( const QString& pathStr );
    void
    reconnect();
    void
    setFile();
    bool
    setModelDirectory( const QString& path );

    UrlWidget*                urlWidget;
    PathWidget*               pathWidget;
    QLineEdit*                fileInput;
    QTreeView*                tree;
    RemoteFileSystemModel*    model;
    FileSortFilterProxyModel* proxy;
    QString                   filename;
    QString                   previousPath = "";
    void
    disconnect();
};
}
#endif // REMOTEFILEDIALOG_H
